/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkMeanImageFilter_h
#define sitkMeanImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class MeanImageFilter
\brief Applies an averaging filter to an image.

Computes an image where a given pixel is the mean value of the the pixels in a neighborhood about the corresponding input pixel.

A mean filter is one of the family of linear filters.

\see Image 

\see Neighborhood 

\see NeighborhoodOperator 

\see NeighborhoodIterator 

\par Wiki Examples:

\li All Examples 

\li Mean filter an image
\sa itk::simple::Mean for the procedural interface
\sa itk::MeanImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT MeanImageFilter : public ImageFilter<1> {
    public:
      typedef MeanImageFilter Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      MeanImageFilter();

      /** Destructor */
      ~MeanImageFilter();

      /** Define the pixels types supported by this filter */
      typedef BasicPixelIDTypeList  PixelIDTypeList;



      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetRadius ( const std::vector<unsigned int> & Radius ) { this->m_Radius = Radius; return *this; }

      /** Set the values of the Radius vector all to value */
      SITK_RETURN_SELF_TYPE_HEADER SetRadius( unsigned int value ) { this->m_Radius = std::vector<unsigned int>(3, value); return *this; }


      /**
       */
        std::vector<unsigned int> GetRadius() const { return this->m_Radius; }
      /** Name of this class */
      std::string GetName() const { return std::string ("MeanImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input image */
      Image Execute ( const Image& image1 );


      /** Execute the filter on the input image with the given parameters */
      Image Execute ( const Image& image1, const std::vector<unsigned int> & radius );


    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)( const Image& image1 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1 );

      /** Dispatched methods which calls ExecuteInteral on each component */
      template <class TImageType> Image ExecuteInternalVectorImage ( const Image& image );

      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;
      friend struct detail::ExecuteInternalVectorImageAddressor<MemberFunctionType>;
      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;


      /*  */
      std::vector<unsigned int>  m_Radius;
    };



    /**
     * \brief Applies an averaging filter to an image.
     *
     * This function directly calls the execute method of MeanImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::MeanImageFilter for the object oriented interface
     */
     SITKBasicFilters_EXPORT Image Mean ( const Image& image1, const std::vector<unsigned int> & radius = std::vector<unsigned int>(3, 1) );

  }
}
#endif
