/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkSpeckleNoiseImageFilter_h
#define sitkSpeckleNoiseImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class SpeckleNoiseImageFilter
\brief Alter an image with speckle (multiplicative) noise.

The speckle noise follows a Gamma distribution of mean 1 and standard deviation provided by the user. The noise is proportional to the pixel intensity.

\author Gaetan Lehmann

This code was contributed in the Insight Journal paper "Noise
Simulation". https://hdl.handle.net/10380/3158
\sa itk::simple::SpeckleNoise for the procedural interface
\sa itk::SpeckleNoiseImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT SpeckleNoiseImageFilter : public ImageFilter<1> {
    public:
      typedef SpeckleNoiseImageFilter Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      SpeckleNoiseImageFilter();

      /** Destructor */
      ~SpeckleNoiseImageFilter();

      /** Define the pixels types supported by this filter */
      typedef BasicPixelIDTypeList  PixelIDTypeList;



      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetStandardDeviation ( double StandardDeviation ) { this->m_StandardDeviation = StandardDeviation; return *this; }

      /**
       */
        double GetStandardDeviation() const { return this->m_StandardDeviation; }

      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetSeed ( uint32_t Seed ) { this->m_Seed = Seed; return *this; }

      /**
       */
        uint32_t GetSeed() const { return this->m_Seed; }
      /** Name of this class */
      std::string GetName() const { return std::string ("SpeckleNoiseImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input image */
      Image Execute ( const Image& image1 );


      /** Execute the filter on the input image with the given parameters */
      Image Execute ( const Image& image1, double standardDeviation, uint32_t seed );


    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)( const Image& image1 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1 );

      /** Dispatched methods which calls ExecuteInteral on each component */
      template <class TImageType> Image ExecuteInternalVectorImage ( const Image& image );

      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;
      friend struct detail::ExecuteInternalVectorImageAddressor<MemberFunctionType>;
      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;


      double  m_StandardDeviation;
      uint32_t  m_Seed;
    };



    /**
     * \brief Alter an image with speckle (multiplicative) noise.
     *
     * This function directly calls the execute method of SpeckleNoiseImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::SpeckleNoiseImageFilter for the object oriented interface
     */
     SITKBasicFilters_EXPORT Image SpeckleNoise ( const Image& image1, double standardDeviation = 1.0, uint32_t seed = (uint32_t) itk::simple::sitkWallClock );

  }
}
#endif
